// SPDX-FileCopyrightText: Copyright © DUNE-FUFEM Project contributors, see file AUTHORS.md
// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception OR LGPL-3.0-or-later

#ifndef DUNE_FUFEM_GEOMETRY_CONVEXPOLYHEDRON_HH
#define DUNE_FUFEM_GEOMETRY_CONVEXPOLYHEDRON_HH

#include <dune/common/rangeutilities.hh>

template <class Coordinate> struct ConvexPolyhedron {
  std::vector<Coordinate> vertices;

  template <class DynamicVector>
  Coordinate barycentricToGlobal(DynamicVector const &b) const {
    assert(b.size() == vertices.size());
    Coordinate r(0);
    for (size_t i = 0; i < b.size(); i++)
      for(auto j : Dune::range(r.size()))
        r[j] += b[i]*vertices[i][j];
    return r;
  }

  template <class DynamicVector>
  void sanityCheck(DynamicVector const &b, double tol) const {
    double sum = 0;
    for (size_t i = 0; i < b.size(); i++) {
      if (b[i] < -tol)
        DUNE_THROW(Dune::Exception, "No barycentric coords " << b[1] << " nr. "
                                                             << i);
      sum += b[i];
    }
    if (sum > 1 + tol)
      DUNE_THROW(Dune::Exception, "No barycentric coords, sum: " << sum);
  }
};
#endif
